expandCore<-function(coreData,V_mR,V_miR,ppimatrix,C){
  #all cmfm core mRNA neibors
  neiborList<-getMRNANeiborList(coreData,V_mR,ppimatrix)
  
  #add mRNA
  addMRNA(coreData,neiborList,V_mR,V_miR,ppimatrix,C)
}


getMRNANeiborList<-function(coreData,V_mR,ppimatrix){
  
  cmfmCore<-coreData$cmfmCore
  neiborList<-c()
  for(i in 1:length(cmfmCore)){
    vmR<-V_mR[[cmfmCore[i]]]
    neibor<-c()
    for(j in 1:length(vmR)){
      mneibor<-ppimatrix[[as.integer(vmR[j])]]
      neibor<-union(neibor,mneibor)
    }
    #remove same node
    neibor<-setdiff(neibor,vmR)
    neiborList<-c(neiborList,list(neibor))
  }
  
  neiborList
}  




addMRNA<-function(coreData,neiborlist,V_mR,V_miR,ppimatrix,C){
  clvmr<-coreData$clvmr
  clvmir<-coreData$clvmir
  cdens_vmr<-coreData$cdens_vmr
  cdens_vmir<-coreData$cdens_vmir
  idensmodule<-coreData$cmfmCore
  
 
  for(i in 1:length(idensmodule)){
    imodule<-as.integer(idensmodule[i])
    mneibor<-neiborlist[[i]]
    for(j in 1:length(mneibor)){
      
      vlength<-length(V_mR[[imodule]])
      oldlen<-clvmr[[imodule]]
      #get a neighbour node
      gene<-as.integer(mneibor[[j]])
      
      #calculate its PPI density
      gneibor<-ppimatrix[[gene]]
      len<-length(intersect(gneibor,V_mR[[imodule]]))
      newlen<-oldlen+len
      dens<-(newlen*2)/((vlength+1)*vlength)
      
      ##calculate its regulation density 
      nmirna<-length(V_miR[[imodule]])
      nmrna<-vlength+1
      
      tmiRpos<-which(C[gene,]>0)
      
      #The remainder is the number of rows
      tmiR<-tmiRpos%%nrow(C)
      #the last one
      tmiR[tmiR==0]<-nrow(C)
      
      
      #calculate new miRNA-mRNA edge number
      newedge<-length(intersect(tmiR,V_miR[[imodule]]))
      midens<-(clvmir[imodule]+newedge)/(nmirna*nmrna)
      #more than the old one
      if(midens+dens>cdens_vmr[imodule]+cdens_vmir[imodule]){
        #update edge number
        clvmir[imodule]<-clvmir[imodule]+newedge
        clvmr[imodule]<-newlen
        #update density
        cdens_vmir[imodule]<-midens
        cdens_vmr[imodule]<-dens
        #update mRNA set
        V_mR[[imodule]]<-union(V_mR[[imodule]],gene)
      }
    }
  }
  

  newCMFM<-matrix(c(idensmodule,V_miR[idensmodule],clvmir[idensmodule],V_mR[idensmodule], clvmr[idensmodule], cdens_vmir[idensmodule], cdens_vmr[idensmodule]),byrow = F,ncol=7)
  colnames(newCMFM)<-c("module","miRNA","regNum","mRNA","ppiNum","regDens","ppiDens")
  newCMFM
}

