source("R/utils.R")
source("R/getCore.R")
source("R/addMRNA.R")
source("R/addMiRNA.R")

###########################################
# run lasso to get miRNA-mRNA network matrix

W<-lasso(C,X,Z)

#keep original names of miRNA and mRNA
miRNAs<-colnames(W)
mRNAs<-rownames(W)

############################################
#save miRNA-mRNA interaction in W to file
#format: mRNA miRNA.one edge,one line

filename<-"biclique/test1.bigraph"
procData(W,filename)

###########################################
#read test1.bigraph，get all maxiaml biclique and write to test2.txt
library(Rcpp)

#load C++ function
sourceCpp("biclique/biclique.cpp")

#1 represents test1.bigraph
biclique(1)

###########################################
#get all maximal bicliques
# first part are mRNAs, next are miRNAs

filename<-"test2.txt"

#raw cmfm cores
rawCore<-readFile(filename)
V_mR<-rawCore[[1]]
V_miR<-rawCore[[2]]

############################################
##build PPI matrix

ppi<-read.csv("data/ppi.csv")
ppimatrix<-buildPPIMatrix(ppi,mRNAs,C)

##########################################
#get cmfm core
 
coreData<-getCore(V_mR,V_miR,ppimatrix)

###########################################

#add mRNA to expand cmfm
newCMFM<-expandCore(coreData,V_mR,V_miR,ppimatrix)


##########################################
#add miRNA to get final cmfm
finalCMFM<-getFinalCMFM(V_mR,V_miR,newCMFM)


##########################################
# output the result

filename<-"data/result/ov(mbcfm).csv"
writeCMFMToFile(newCMFM,miRNAs,mRNAs,filename)


